﻿//////////////////////////////////////////////
// MainSystem.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class RenderContext ;
	class Renderer ;
}

namespace nkWinUi
{
	class MainSystem ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT MainSystem final : public nkCommon::SingletonClass<nkGraphics::MainSystem>
	{
		friend class nkCommon::SingletonClass<nkGraphics::MainSystem> ;

		private :
		
			// Attributes
			// Ui system used
			nkWinUi::MainSystem* _uiSystem ;
			// Render system used
			Renderer* _renderer ;

			// Witnesses
			bool _initialized ;
			bool _hasRunToContinue ;

		private :

			// Functions
			// Constructor
			MainSystem () ;
		
		public :
		
			// Destructor
			~MainSystem () ;

			// Getters
			nkWinUi::MainSystem* getUiSystem () const ;
			Renderer* getRenderer () const ;
			bool isInitialized () const ;
			bool getHasRunToContinue () const ;

			// Setters
			void setHasRunToContinue (bool value) ;

			// To start
			bool autoConfigure () ;
			bool initialize () ;
			void run (RenderContext* context) ;

			// To end
			void shutdown () ;

			// Step by step
			bool frame (RenderContext* context) ;
			void tickForNewFrame (RenderContext* context) ;
	} ;
}