﻿//////////////////////////////////////////////
// UniquePtr.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// Standards
#include <memory>

/// Class ------------------------------------

namespace nkMemory
{
	template <typename T>
	class UniquePtr
	{
		private :

			// Attributes
			// Base buffer
			T* _data ;

		public :

			// Constructors
			UniquePtr ()
			:	_data (nullptr)
			{
				// Nothing to do
			}

			UniquePtr (T* data)
			:	_data (data)
			{
				// Nothing to do
			}

			template <typename U, typename = std::enable_if_t<std::is_base_of_v<T, U>>>
			UniquePtr (U* data)
			:	_data (data)
			{
				// Nothing to do
			}

			UniquePtr (const UniquePtr<T>& other) = delete ;

			UniquePtr (UniquePtr<T>&& other)
			:	_data (other._data)
			{
				// Reset input
				other._data = nullptr ;
			}

			// Destructor
			~UniquePtr ()
			{
				delete _data ;
				_data = nullptr ;
			}

		public :

			// Getters
			T* get () const
			{
				return _data ;
			}

			bool empty () const
			{
				return !_data ;
			}

		public :

			// Management
			void release ()
			{
				_data = nullptr ;
			}

			void reset (T* data = nullptr)
			{
				delete _data ;
				_data = data ;
			}

		public :

			// Operators
			T& operator* ()
			{
				return *_data ;
			}

			const T& operator* () const
			{
				return *_data ;
			}
			
			UniquePtr<T>& operator= (const UniquePtr<T>& other) = delete ;

			UniquePtr<T>& operator= (UniquePtr<T>&& other)
			{
				delete _data ;
				_data = other._data ;

				other._data = nullptr ;

				return *this ;
			}

			operator bool () const
			{
				return _data ;
			}

			bool operator! () const
			{
				return !_data ;
			}

			operator std::unique_ptr<T> ()
			{
				T* data = _data ;
				_data = nullptr ;

				return std::unique_ptr<T>(data) ;
			}

			template <typename U, typename = std::enable_if_t<std::is_base_of_v<U, T>>>
			operator std::unique_ptr<U> ()
			{
				T* data = _data ;
				_data = nullptr ;

				return std::unique_ptr<T>(data) ;
			}

		public :

			// Templated constructors
			UniquePtr (std::unique_ptr<T> stdPtr)
			:	_data (stdPtr.get())
			{
				// Reset source
				stdPtr.release() ;
			}

			template <typename U, typename = std::enable_if_t<std::is_base_of_v<T, U>>>
			UniquePtr (std::unique_ptr<U> stdPtr)
			:	_data (stdPtr.get())
			{
				// Reset source
				stdPtr.release() ;
			}
	} ;
}