﻿//////////////////////////////////////////////
// TreeView.h
//
//////////////////////////////////////////////

/// Defines / macro --------------------------

#pragma once

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/BufferCast.h>
#include <NilkinsMemory/Containers/String.h>

// Standards
#include <functional>

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT TreeView : public Component
	{
		public :

			class DLL_WINUI_EXPORT Item
			{
				protected :
						
					// Attributes
					// Children
					nkMemory::BufferCast<Item*> _children ;

					// Its text
					nkMemory::String _text ;

					// Parents
					TreeView* _parentTree ;
					Item* _parentItem ;
					Item* _precedingItem ;

					// Data
					void* _attachedData ;

				public :

					// Constructor, destructor
					Item () ;
					virtual ~Item () ;

					// Children
					Item* addChild (Item* child) ;
					Item* getChild (unsigned int index) const ;
					void disavowChild (Item* child) ;

					// Text
					void setText (const nkMemory::StringView& text) ;
					nkMemory::StringView getText () const ;

					// Parents
					void setParentTree (TreeView* parent) ;
					TreeView* getParentTree () const ;
					void setParentItem (Item* item) ;
					Item* getParentItem () const ;
					void setPrecedingItem (Item* item) ;
					Item* getPrecedingItem () const ;

					// Data
					void setAttachedData (void* data) ;
					void* getAttachedData () const ;

					// Export
					void exportIntrospection (nkExport::Node* rootNode) ;
			} ;

		protected :

			// Functions
			virtual Item* _allocateItem () const = 0 ;

		protected :

			// Attributes
			// List of items
			nkMemory::BufferCast<Item*> _items ;

			// Callbacks
			std::function<void (TreeView*)> _selectionCallback ;

			// Flags
			bool _selectOnRightClick ;
		
		public :
		
			// Constructor, destructor
			TreeView (MainSystem* parentSystem) ;
			virtual ~TreeView () ;

			// Getters
			Item* getItemFromData (void* data) const ;
			virtual Item* getSelectedItem () = 0 ;
			virtual Item* getItemAtAbsoluteCursor (const nkMaths::Point& absCoords) = 0 ;
			virtual Item* getItemAtRelativeCursor (const nkMaths::Point& relCoords) = 0 ;
			bool getSelectOnRightClick () const ;
			std::function<void (TreeView*)> getSelectionCallback () const ;

			// Setters
			void setSelectionOnRightClick (bool value) ;
			void setSelectionCallback (std::function<void (TreeView*)> value) ;

			// Items management
			virtual Item* insertItem (const nkMemory::StringView& text, void* data, Item* parent = nullptr, Item* afterThisOne = nullptr) ;
			virtual void deleteItem (Item* toDelete, bool keepChildrenOnUpper = false) ;
			virtual void changeItemLabel (Item* toChange, const nkMemory::StringView& label) ;
			virtual void clearItems () ;

			// Selection
			void selectItemFromItsData (void* data) ;
			virtual void selectItem (Item* item) = 0 ;
			void selectItemFromAbsoluteCursor (const nkMaths::Point& absCoords) ;
			void selectItemFromRelativeCursor (const nkMaths::Point& relCoords) ;

			// Import / Export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}