﻿//////////////////////////////////////////////
// MainSystem.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkWinUi
{
	class ComponentManager ;
	class CursorManager ;
	class FontManager ;
	class InputSystem ;
	class LogManager ;
	class ThemeManager ;
	class WindowingSystem ;
}

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/MaybeSingletonClass.h>

// Standards
#include <functional>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT MainSystem final : public nkCommon::MaybeSingletonClass<MainSystem>
	{
		private :

			// Functions
			// Start
			bool _createDefaultTheme () const ;

		private :
		
			// Attributes
			// Windowing system
			WindowingSystem* _windowingSystem ;

			// Components
			ComponentManager* _componentManager ;
			CursorManager* _cursorManager ;
			FontManager* _fontManager ;
			InputSystem* _inputSystem ;
			LogManager* _logManager ;
			ThemeManager* _themeManager ;

			// Callbacks
			std::function<bool ()> _runCallback ;

			// Flags
			bool _initialized ;
		
		public :

			// Constructor, destructor
			MainSystem () ;
			~MainSystem () ;

			// Getters
			WindowingSystem* getWindowingSystem () const ;
			ComponentManager* getComponentManager () const ;
			CursorManager* getCursorManager () const ;
			FontManager* getFontManager () const ;
			InputSystem* getInputSystem () const ;
			LogManager* getLogManager () const ;
			ThemeManager* getThemeManager () const ;
			std::function<bool ()> getRunCallback () const ;
			bool isInitialized () const ;

			// Setters
			void setRunCallback (std::function<bool ()> callback) ;

			// Start
			bool initialize () ;

			// Running
			void run () ;
			void tick () ;

			// End
			void prepareForShutdown () ;
			void shutdown () ;

			// Dummy event to trigger behaviour
			void notifyEvent () ;
	} ;
}